package main

import (
	"context"
	"crypto/sha256"
	"gitea.illuad.fr/adrien/middleman/command"
	"github.com/rs/zerolog/log"
	"github.com/urfave/cli/v3"
	"golang.org/x/sync/errgroup"
	"os"
	"os/signal"
	"runtime/debug"
	"syscall"
)

type noCommit struct {
	message []byte
}

func (nc noCommit) String() string {
	if nc.message == nil {
		nc.message = []byte("not tied to a commit") // 6344da387a21711b36a91c2fd55f48a026b2be8b5345cbbf32a31f5c089f4d75
	}
	sum := sha256.Sum256(nc.message)
	return string(sum[:])
}

const appName = "middleman"

var (
	version = "dev"
	commit  = noCommit{}.String()
)

func main() {
	info, ok := debug.ReadBuildInfo()
	if ok {
		if info.Main.Version != "" {
			version = info.Main.Version
			commit = info.Main.Sum
		}
	}
	// Note: os.Kill/syscall.SIGKILL (SIGKILL) signal cannot be caught or ignored.
	ctx, cancel := signal.NotifyContext(context.Background(), os.Interrupt, syscall.SIGTERM)
	defer cancel()
	var group *errgroup.Group
	group, ctx = errgroup.WithContext(ctx)
	app := cli.Command{
		Name:           appName,
		Usage:          "Securely mount the Docker socket: apply fine-grained access control to Docker socket HTTP requests",
		Version:        version + "-" + commit,
		DefaultCommand: command.ServeCmd,
		Commands: []*cli.Command{
			command.Serve(group),
			command.Healthcheck(group),
		},
		Authors: []any{
			"Adrien <contact@illuad.fr>",
		},
		UseShortOptionHandling: true,
	}
	group.Go(func() error {
		return app.Run(ctx, os.Args)
	})
	if err := group.Wait(); err != nil {
		log.Err(err).Send()
	}
}
